#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <GL/gl.h>
#include <GL/glut.h>
#include <vrml97/Doc.h>
#include <vrml97/System.h>
#include <vrml97/VrmlScene.h>
extern "C" {
#include <AR/vrml97.h>
}

#define M_PI                  (3.14159265359 )

#include "vrml97int.h"

#define  AR_VRML97_MAX   30

static arVrml97Viewer  *viewer[AR_VRML97_MAX];
static int              init = 1;

static int              vrID        = -1;
static int              bg_flag     = 0;
static int              fog_flag    = 0;
static int              avatar_flag = 0;
static int              lastButton  = 0;

static double           s_modelview[16];
static double           s_projection[16];

static void specialKey(int key, int x, int y);
static void mouse(int button, int state, int x, int y);
static void motion(int x, int y);
static void passiveMotion(int x, int y);
static char *get_buff( char *buf, int n, FILE *fp );
static void recalc_pos( int *x, int *y );
static int load_light( arVrml97Viewer *viewer, char *buf, int n, FILE *fp);

int arVrml97InsertAvatar( int id, int avatar_id, double scale, double trans_mat[16] )
{
    if( viewer[id] == NULL )        return -1;
    if( viewer[avatar_id] == NULL ) return -1;

    viewer[id]->insertAvatar( avatar_id, scale, trans_mat );

    return 0;
}

int arVrml97ClickButton( int x, int y )
{
    glPushAttrib(GL_ALL_ATTRIB_BITS);

    passiveMotion(x, y);
    mouse( GLUT_LEFT_BUTTON, GLUT_DOWN, x, y);
    mouse( GLUT_LEFT_BUTTON, GLUT_UP, x, y);

    glPopAttrib();

    return 0;
}


int arVrml97ChangeVR( int id )
{
    if( id < 0 ) {
        if( vrID >= 0 ) {
            viewer[vrID]->arVrml97ChangeARMode( 1 );
        }
        vrID = -1;

        glPopAttrib();
        glutSpecialFunc( NULL );
        glutMouseFunc( NULL );
        glutMotionFunc( NULL );
        glutPassiveMotionFunc( NULL );
    }
    else {
        if( viewer[id] == NULL ) return -1;

        if( vrID >= 0 && vrID != id ) {
            viewer[vrID]->arVrml97ChangeARMode( 1 );
        }
        viewer[id]->arVrml97ChangeARMode( 0 );
        vrID = id;
        arVrml97ResetUserPos();

        glutSpecialFunc( specialKey);
        glutMouseFunc( mouse);
        glutMotionFunc( motion);
        glutPassiveMotionFunc( passiveMotion);
        glPushAttrib(GL_ALL_ATTRIB_BITS);
    }

    return 0;
}

int arVrml97ResetUserPos( void )
{
    if( vrID < 0 ) return -1;
    viewer[vrID]->resetUserNavigation();

    return 0;
}

int arVrml97MoveUserPos( float pos[3] )
{
    if( vrID < 0 ) return -1;

    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();
    glTranslatef( pos[0], pos[1], pos[2] );
    glMultMatrixd( viewer[vrID]->userPosMat );
    glGetDoublev( GL_MODELVIEW_MATRIX, viewer[vrID]->userPosMat );
    glPopMatrix();

    return 0;
}

int arVrml97MoveUserPosMat( double mat[16] )
{
    if( vrID < 0 ) return -1;

    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadMatrixd( mat );
    glMultMatrixd( viewer[vrID]->userPosMat );
    glGetDoublev( GL_MODELVIEW_MATRIX, viewer[vrID]->userPosMat );
    glPopMatrix();

    return 0;
}

int arVrml97RotateUserPos( float rot[4] )
{
    if( vrID < 0 ) return -1;

    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();
    glRotatef( rot[0], rot[1], rot[2], rot[3] );
    glMultMatrixd( viewer[vrID]->userPosMat );
    glGetDoublev( GL_MODELVIEW_MATRIX, viewer[vrID]->userPosMat );
    glPopMatrix();

    return 0;
}

int arVrml97GetUserPosMat( int id, double mat[16] )
{
    if( id < 0 ) return -1;

    for( int i = 0; i < 16; i++ ) {
        mat[i] = viewer[id]->userPosMat[i];
    }

    return 0;
}

int arVrml97SetUserPos( float rot[4], float pos[3] )
{
    if( vrID < 0 ) return -1;

    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();
    glRotatef( rot[0], rot[1], rot[2], rot[3] );
    glTranslatef( pos[0], pos[1], pos[2] );
    glGetDoublev( GL_MODELVIEW_MATRIX, viewer[vrID]->userPosMat );
    glPopMatrix();

    return 0;
}

int arVrml97GetAvatarPosMat( int id, double mat[16] )
{
    if( id < 0 ) return -1;

    for( int i = 0; i < 16; i++ ) {
        mat[i] = viewer[id]->userPosMat2[i];
    }

    return 0;
}

int arVrml97LoadFile( char *file )
{
    FILE             *fp;
    VrmlScene        *vrmlScene;
    char             buf[256], buf1[256];
    int              id;
    int              i;

    if( init ) {
        for( i = 0; i < AR_VRML97_MAX; i++ ) viewer[i] = NULL;
        init = 0;
    }
    for( i = 0; i < AR_VRML97_MAX; i++ ) {
        if( viewer[i] == NULL ) break;
    }
    if( i == AR_VRML97_MAX ) return -1;
    id = i;


    if( (fp=fopen(file, "r")) == NULL ) return -1;

    get_buff(buf, 256, fp);
    if( sscanf(buf, "%s", buf1) != 1 ) {fclose(fp); return -1;}

    vrmlScene = new VrmlScene( buf1, buf1 );
    if( vrmlScene == NULL ) {fclose(fp); return -1;}

    viewer[id] = new arVrml97Viewer( vrmlScene );
    if( viewer[i] == NULL ) {
        delete vrmlScene;
        fclose(fp);
        return -1;
    }
    strcpy( viewer[i]->filename, buf1 );

    get_buff(buf, 256, fp);
    if( sscanf(buf, "%lf %lf %lf", &viewer[id]->translation[0],
        &viewer[id]->translation[1], &viewer[id]->translation[2]) != 3 ) {
        delete viewer[id];
        viewer[id] = NULL;
        fclose(fp);
        return -1;
    }

    get_buff(buf, 256, fp);
    if( sscanf(buf, "%lf %lf %lf %lf", &viewer[id]->rotation[0],
        &viewer[id]->rotation[1], &viewer[id]->rotation[2], &viewer[id]->rotation[3]) != 4 ) {
        delete viewer[id];
        viewer[id] = NULL;
        fclose(fp);
        return -1;
    }

    get_buff(buf, 256, fp);
    if( sscanf(buf, "%lf %lf %lf", &viewer[id]->scale[0], &viewer[id]->scale[1],
               &viewer[id]->scale[2]) != 3 ) {
        delete viewer[id];
        viewer[id] = NULL;
        fclose(fp);
        return -1;
    }

    for(;;) {
        if( get_buff(buf, 256, fp) == NULL ) break;

        sscanf(buf, "%s", buf1);
        if( strcmp(buf1, "LIGHT") == 0 ) {
            if( load_light(viewer[id], buf, 256, fp) < 0 ) break;
        }
    }
    fclose(fp);

    return id;
}

int arVrml97Free( int id )
{
    if( viewer[id] == NULL ) return -1;

    delete viewer[id];
    viewer[id] = NULL;

    if( vrID == id ) {
        vrID = -1;

        glPopAttrib();
        glutSpecialFunc( NULL );
        glutMouseFunc( NULL );
        glutMotionFunc( NULL );
        glutPassiveMotionFunc( NULL );
    }

    return 0;
}

int arVrml97Reset( int id )
{
    if( viewer[id] == NULL ) return -1;

#if 0
    delete viewer[id]->vrmlScene;
    viewer[id]->vrmlScene = new VrmlScene( viewer[id]->filename, viewer[id]->filename );
    if( viewer[id]->vrmlScene == NULL ) {
        viewer[id] = NULL;
        if( vrID == id ) {
            vrID = -1;
            glPopAttrib();
            glutSpecialFunc( NULL );
            glutMouseFunc( NULL );
            glutMotionFunc( NULL );
            glutPassiveMotionFunc( NULL );
        }
        return -1;
    }
#endif

    viewer[id]->vrmlScene->resetTimer();

    return 0;
}

int arVrml97TimerUpdate()
{
    int     i;

    for( i = 0; i < AR_VRML97_MAX; i++ ) {
        if( viewer[i] == NULL ) continue;
        viewer[i]->timerUpdate();
    }

    return 0;
}

int arVrml97Draw( int id )
{
    if( viewer[id] == NULL ) return -1;

    viewer[id]->render();

    return 0;
}

int arVrml97SetFog( int flag )
{
    fog_flag = flag;

    return 0;
}

int arVrml97SetBG( int flag )
{
    bg_flag = flag;

    return 0;
}

int arVrml97Viewer::insertAvatar( int avatar_id, double scale, double trans_mat[16] )
{
    int     i, j;

    for( i = 0; i < AR_AVATAR_MAX; i++ ) {
        if( avatar[i].id < 0 ) break;
    }
    if( i == AR_AVATAR_MAX ) return -1;

    avatar[i].id    = avatar_id;
    avatar[i].scale = scale;
    for( j = 0; j < 16; j++ ) {
        avatar[i].trans[j] = trans_mat[j];
    }

    return 0;
}

arVrml97Viewer::arVrml97Viewer(VrmlScene *scene) : ViewerOpenGL(scene)
{
    int     i;

    vrmlScene = scene;
    vrmlScene->arView = 1;
    arView = 1;

    for( i = 0; i < AR_AVATAR_MAX; i++ ) {
        avatar[i].id = -1;
    }

    for( i = 0; i < AR_LIGHT_MAX; i++ ) {
        dir_light[i].active   = 0;
        point_light[i].active = 0;
        spot_light[i].active  = 0;
    }

    translation[0] = 0.0;
    translation[1] = 0.0;
    translation[2] = 0.0;

    rotation[0] = 0.0;
    rotation[1] = 0.0;
    rotation[2] = 0.0;
    rotation[3] = 0.0;

    scale[0] = 0.0;
    scale[1] = 0.0;
    scale[2] = 0.0;
}

arVrml97Viewer::~arVrml97Viewer()
{
    delete vrmlScene;
}

int arVrml97Viewer::arVrml97ChangeARMode( int flag )
{
    arView = flag;
    vrmlScene->arView = flag;

    if( flag == 0 ) {
        resetUserNavigation();
    }

    return 0;
}

void arVrml97Viewer::timerUpdate()
{
    update( 0.0 );
}

void arVrml97Viewer::render()
{
    int   flag_backup = arView;

    if( avatar_flag == 1 ) {
        arView = 1;
        vrmlScene->arView = 1;
    }

    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glTranslated( translation[0], translation[1], translation[2] );
    if( rotation[0] != 0.0 ) {
        glRotated( rotation[0], rotation[1], rotation[2], rotation[3] );
    }
    glScaled( scale[0], scale[1], scale[2] );

    if( avatar_flag == 0 ) {
        glPushAttrib(GL_ALL_ATTRIB_BITS);
        glEnable(GL_NORMALIZE);
        glEnable(GL_DEPTH_TEST);
        glDepthFunc(GL_LEQUAL);
        glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
#if 0
        glEnable(GL_STENCIL_TEST);
        glStencilFunc(GL_ALWAYS, 1, 1);
        glStencilOp(GL_KEEP, GL_KEEP, GL_REPLACE);
#endif
    }

    glGetDoublev(GL_PROJECTION_MATRIX, s_projection);
    glGetDoublev(GL_MODELVIEW_MATRIX,  s_modelview);


    if( avatar_flag == 0 ) {
        redraw();

        avatar_flag = 1;
        for( int i = 0; i < AR_AVATAR_MAX; i++ ) {
            if( avatar[i].id < 0 ) continue;

            glMatrixMode(GL_MODELVIEW);
            glPushMatrix();
            glMultMatrixd( avatar[i].trans );
            glScalef( avatar[i].scale, avatar[i].scale, avatar[i].scale );
            viewer[avatar[i].id]->render();
            glPopMatrix();
            avatar[i].id = -1;
        }
        avatar_flag = 0;
    }
    else {
        redraw_avatar();
    }

    glMatrixMode(GL_PROJECTION);
    glLoadMatrixd(s_projection);
    glMatrixMode(GL_MODELVIEW);
    glLoadMatrixd(s_modelview);
    if( avatar_flag == 0 ) {
        glPopAttrib();
    }

    glMatrixMode(GL_MODELVIEW);
    glPopMatrix();

    if( avatar_flag == 1 ) {
        arView = flag_backup;
        vrmlScene->arView = flag_backup;
    }
}

int arVrml97Viewer::arVrml97InsertDirLight( float ambient, float intensity,
                                            float rgb[3], float direction[3] )
{
    int     i;

    for( i = 0; i < AR_LIGHT_MAX; i++ ) {
        if( dir_light[i].active == 0 ) break;
    }
    if( i == AR_LIGHT_MAX ) return -1;

    dir_light[i].active    = 1;
    dir_light[i].ambient   = ambient;
    dir_light[i].intensity = intensity;
    dir_light[i].rgb[0]    = rgb[0];
    dir_light[i].rgb[1]    = rgb[1];
    dir_light[i].rgb[2]    = rgb[2];
    dir_light[i].direction[0] = direction[0];
    dir_light[i].direction[1] = direction[1];
    dir_light[i].direction[2] = direction[2];

    return 0;
}

int arVrml97Viewer::arVrml97InsertPointLight( float ambient, float attenuation[3],
          float rgb[3], float intensity, float location[3], float radius )
{
    int     i;

    for( i = 0; i < AR_LIGHT_MAX; i++ ) {
        if( point_light[i].active == 0 ) break;
    }
    if( i == AR_LIGHT_MAX ) return -1;

    point_light[i].active    = 1;
    point_light[i].ambient   = ambient;
    point_light[i].attenuation[0] = attenuation[0];
    point_light[i].attenuation[1] = attenuation[1];
    point_light[i].attenuation[2] = attenuation[2];
    point_light[i].rgb[0]    = rgb[0];
    point_light[i].rgb[1]    = rgb[1];
    point_light[i].rgb[2]    = rgb[2];
    point_light[i].intensity = intensity;
    point_light[i].location[0] = location[0];
    point_light[i].location[1] = location[1];
    point_light[i].location[2] = location[2];
    point_light[i].radius = radius;

    return 0;
}

int arVrml97Viewer::arVrml97InsertSpotLight( float ambient, float attenuation[3],
          float beamWidth, float rgb[3], float cutOffAngle, float direction[3],
          float intensity, float location[3], float radius )
{
    int     i;

    for( i = 0; i < AR_LIGHT_MAX; i++ ) {
        if( spot_light[i].active == 0 ) break;
    }
    if( i == AR_LIGHT_MAX ) return -1;

    spot_light[i].active    = 1;
    spot_light[i].ambient   = ambient;
    spot_light[i].attenuation[0] = attenuation[0];
    spot_light[i].attenuation[1] = attenuation[1];
    spot_light[i].attenuation[2] = attenuation[2];
    spot_light[i].beamWidth = beamWidth;
    spot_light[i].rgb[0]    = rgb[0];
    spot_light[i].rgb[1]    = rgb[1];
    spot_light[i].rgb[2]    = rgb[2];
    spot_light[i].cutOffAngle = cutOffAngle;
    spot_light[i].direction[0] = direction[0];
    spot_light[i].direction[1] = direction[1];
    spot_light[i].direction[2] = direction[2];
    spot_light[i].intensity = intensity;
    spot_light[i].location[0] = location[0];
    spot_light[i].location[1] = location[1];
    spot_light[i].location[2] = location[2];
    spot_light[i].radius = radius;

    return 0;
}



void arVrml97Viewer::setViewpoint(float *position,
                                  float *orientation,
                                  float fieldOfView,
                                  float avatarSize,
                                  float visibilityLimit)
{
    int     i;

    if( arView == 0 ) {
        GLint viewport[4];

        glGetIntegerv(GL_VIEWPORT, viewport);
        ViewerOpenGL::resize(640,480);
        glViewport(viewport[0], viewport[1], viewport[2], viewport[3]);

        ViewerOpenGL::setViewpoint( position, orientation, fieldOfView,
                                    avatarSize, 2.0*visibilityLimit);
        return;
    }

    glMatrixMode(GL_PROJECTION);
    glLoadMatrixd( s_projection );

    glMatrixMode(GL_MODELVIEW);
    glLoadMatrixd( s_modelview );

    if ( vrmlScene->headlightOn() ) {
        float rgb[3] = { 1.0, 1.0, 1.0 };
        float xyz[3] = { 0.0, 0.0, 1.0 };
        float ambient = 0.3;

        glPushMatrix();
        glLoadIdentity();
        insertDirLight( ambient, 1.0, rgb, xyz );
        glPopMatrix();
    }

    for( i = 0; i < AR_LIGHT_MAX; i++ ) {
        if( dir_light[i].active ) {
            insertDirLight(dir_light[i].ambient,
                           dir_light[i].intensity,
                           dir_light[i].rgb,
                           dir_light[i].direction);
        }
        if( point_light[i].active ) {
            insertPointLight(point_light[i].ambient,
                             point_light[i].attenuation,
                             point_light[i].rgb,
                             point_light[i].intensity,
                             point_light[i].location,
                             point_light[i].radius);
        }
        if( spot_light[i].active ) {
            insertSpotLight(spot_light[i].ambient,
                            spot_light[i].attenuation,
                            spot_light[i].beamWidth,
                            spot_light[i].rgb,
                            spot_light[i].cutOffAngle,
                            spot_light[i].direction,
                            spot_light[i].intensity,
                            spot_light[i].location,
                            spot_light[i].radius);
        }
    }
}

void arVrml97Viewer::setFog(float * color,
                            float visibilityRange,
                            const char * fogType)
{
    if( arView == 0 || fog_flag == 1 ) {
        ViewerOpenGL::setFog( color, visibilityRange, fogType );
    }
    else {
        glDisable( GL_FOG );
    }
}


Viewer::Object arVrml97Viewer::insertBackground(int nGroundAngles,
                                                float* groundAngle,
                                                float* groundColor,
                                                int nSkyAngles,
                                                float* skyAngle,
                                                float* skyColor,
                                                int *whc,
                                                unsigned char **pixels)
{
    GLuint  glid = 0;
    float   r = 0.0, g = 0.0, b = 0.0, a = 1.0;

    if( arView == 0 ) {
        return ViewerOpenGL::insertBackground( nGroundAngles, groundAngle, groundColor, nSkyAngles,
                                               skyAngle, skyColor, whc, pixels);
    }

    if( bg_flag == 0 ) {
        return (Object) glid;
    }

    // Clear to last sky color
    if (skyColor != 0) {
        r = skyColor[3*nSkyAngles+0];
        g = skyColor[3*nSkyAngles+1];
        b = skyColor[3*nSkyAngles+2];
    }


    // Need to separate the geometry from the transformation so the
    // dlist doesn't have to get rebuilt for every mouse movement...
#if USE_GEOMETRY_DISPLAY_LISTS && 0
    // Don't bother with a dlist if we aren't drawing anything
    if( nSkyAngles > 0 || nGroundAngles > 0 || pixels) )
    {
      glid = glGenLists(1);
      glNewList( glid, GL_COMPILE_AND_EXECUTE );
    }
#endif

    glClearColor( r, g, b, a );
    GLuint mask = GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT;
#if USE_STENCIL_SHAPE && 0
    mask |= GL_STENCIL_BUFFER_BIT;
    glClear( mask );
#else
    glClear( mask );

    // Draw the background as big spheres centered at the view position
    if( nSkyAngles > 0 || nGroundAngles > 0 || pixels ) {
        glDisable( GL_DEPTH_TEST );
        glDisable( GL_LIGHTING );

        glPushMatrix();
/*
        glLoadMatrixd( s_modelview );
*/

        glScalef(1000.,1000.,1000.);

        // Sphere constants
        const int nCirc = 8;              // number of circumferential slices
        const double cd = 2.0 * M_PI / nCirc;

        double heightAngle0, heightAngle1 = 0.0;
        float *c0, *c1 = skyColor;

        for (int nSky=0; nSky<nSkyAngles; ++nSky) {
            heightAngle0 = heightAngle1;
            heightAngle1 = skyAngle[nSky];
            c0 = c1;
            c1 += 3;

            double circAngle0, circAngle1 = 0.0;
            double sha0 = sin(heightAngle0), cha0 = cos(heightAngle0);
            double sha1 = sin(heightAngle1), cha1 = cos(heightAngle1);
            double sca0, cca0;
            double sca1 = sin(circAngle1), cca1 = cos(circAngle1);

            glBegin( GL_QUADS );
            for (int nc=0; nc<nCirc; ++nc) {
                circAngle0 = circAngle1;
                circAngle1 = (nc+1) * cd;
                sca0 = sca1;
                sca1 = sin(circAngle1);
                cca0 = cca1;
                cca1 = cos(circAngle1);
                glColor3fv( c1 );
                glVertex3f( sha1 * cca0, cha1, sha1 * sca0 );
                glVertex3f( sha1 * cca1, cha1, sha1 * sca1 );
                glColor3fv( c0 );
                glVertex3f( sha0 * cca1, cha0, sha0 * sca1 );
                glVertex3f( sha0 * cca0, cha0, sha0 * sca0 );
            }
            glEnd();
        }
        // Ground
        heightAngle1 = M_PI;
        c1 = groundColor;

        for (int nGround=0; nGround<nGroundAngles; ++nGround) {
            heightAngle0 = heightAngle1;
            heightAngle1 = M_PI - groundAngle[nGround];
            c0 = c1;
            c1 += 3;

            double circAngle0, circAngle1 = 0.0;
            double sha0 = sin(heightAngle0), cha0 = cos(heightAngle0);
            double sha1 = sin(heightAngle1), cha1 = cos(heightAngle1);
            double sca0, cca0;
            double sca1 = sin(circAngle1), cca1 = cos(circAngle1);

            glBegin( GL_QUADS );
            for (int nc=0; nc<nCirc; ++nc) {
                circAngle0 = circAngle1;
                circAngle1 = (nc+1) * cd;
                sca0 = sca1;
                sca1 = sin(circAngle1);
                cca0 = cca1;
                cca1 = cos(circAngle1);
                glColor3fv( c1 );
                glVertex3f( sha1 * cca1, cha1, sha1 * sca1 );
                glVertex3f( sha1 * cca0, cha1, sha1 * sca0 );
                glColor3fv( c0 );
                glVertex3f( sha0 * cca0, cha0, sha0 * sca0 );
                glVertex3f( sha0 * cca1, cha0, sha0 * sca1 );
            }
            glEnd();
        }

      // Background textures are drawn on a transparent cube
      if (pixels)
        {
          float v[6][4][3] = {
            {{1,-1,1}, {-1,-1,1}, {-1,1,1}, {1,1,1}},     // Back
            {{-1,-1,1}, {1,-1,1}, {1,-1,-1}, {-1,-1,-1}}, // Bottom
            {{-1,-1,-1}, {1,-1,-1}, {1,1,-1}, {-1,1,-1}}, // Front
            {{-1,-1,1}, {-1,-1,-1}, {-1,1,-1}, {-1,1,1}}, // Left
            {{1,-1,-1}, {1,-1,1}, {1,1,1}, {1,1,-1}},     // Right
            {{-1,1,-1}, {1,1,-1}, {1,1,1}, {-1,1,1}}};    // Top

          glScalef( 0.5, 0.5, 0.5 );

          glEnable( GL_TEXTURE_2D );
          glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );

          int t, lastT = -1;
          for (t=0; t<6; ++t, whc+=3) {
            // Check for non-zero width,height,coords and pixel data
            if (whc[0] && whc[1] && whc[2] && pixels[t])
              {
                // Optimize for the case where the same texture is used
                if (lastT == -1 || pixels[t] != pixels[lastT])
                  insertTexture(whc[0], whc[1], whc[2],
                                false, false, pixels[t],
                                false);  // Don't put the textures in dlists

                lastT = t;
                glBegin( GL_QUADS );
                glTexCoord2f( 0.0, 1.0 );
                glVertex3fv( v[t][0] );
                glTexCoord2f( 1.0, 1.0 );
                glVertex3fv( v[t][1] );
                glTexCoord2f( 1.0, 0.0 );
                glVertex3fv( v[t][2] );
                glTexCoord2f( 0.0, 0.0 );
                glVertex3fv( v[t][3] );
                glEnd();
              }
          }
          glDisable( GL_TEXTURE_2D );
        }

      // Put everything back the way it was
      glPopMatrix();

      glEnable( GL_LIGHTING );
      glEnable( GL_DEPTH_TEST );
    }
#endif

    //endGeometry();
    if (glid) glEndList();

    return (Object) glid;
}

void arVrml97Viewer::wsSetCursor( CursorStyle c)
{
  static int cursorMap[] = { GLUT_CURSOR_INHERIT,
                             GLUT_CURSOR_INFO,
                             GLUT_CURSOR_CYCLE,
                             GLUT_CURSOR_UP_DOWN,
                             GLUT_CURSOR_CROSSHAIR
  };

  glutSetCursor( cursorMap[c] );
}

static char *get_buff( char *buf, int n, FILE *fp )
{
    char *ret, buf1[256];

    for(;;) {
        ret = fgets( buf, n, fp );
        if( ret == NULL ) return(NULL);
        if( sscanf(buf, "%s", buf1) != 1 ) continue;

        if( buf1[0] != '#' ) return(ret);
    }
}

//  More keyboard buttons callback

static void specialKey(int key, int x, int y)
{
    if( viewer[vrID] == NULL ) return;

    recalc_pos( &x, &y );
    ViewerOpenGL::EventInfo e = { ViewerOpenGL::EVENT_KEY_DOWN, 0, x, y };

    switch (key) {
        case GLUT_KEY_HOME:  e.what = ViewerOpenGL::KEY_HOME; break;
        case GLUT_KEY_LEFT:  e.what = ViewerOpenGL::KEY_LEFT; break;
        case GLUT_KEY_UP:    e.what = ViewerOpenGL::KEY_UP; break;
        case GLUT_KEY_RIGHT: e.what = ViewerOpenGL::KEY_RIGHT; break;
        case GLUT_KEY_DOWN:  e.what = ViewerOpenGL::KEY_DOWN; break;
        case GLUT_KEY_PAGE_DOWN: e.what = ViewerOpenGL::KEY_PAGE_DOWN; break;
        case GLUT_KEY_PAGE_UP: e.what = ViewerOpenGL::KEY_PAGE_UP; break;
        default: return;
    }

    viewer[vrID]->input( &e );
}

//  Mouse button callback

static void mouse(int button, int state, int x, int y)
{
    if( viewer[vrID] == NULL ) return;

    recalc_pos( &x, &y );
    ViewerOpenGL::EventInfo e = { ViewerOpenGL::EVENT_MOUSE_CLICK, button, x, y };
    if (state == GLUT_UP) e.event = ViewerOpenGL::EVENT_MOUSE_RELEASE;
    lastButton = button;
    viewer[vrID]->input( &e );
}

//  Mouse drag motion callback (button down)

static void motion(int x, int y)
{
    if( viewer[vrID] == NULL ) return;

    recalc_pos( &x, &y );
    ViewerOpenGL::EventInfo e = { ViewerOpenGL::EVENT_MOUSE_DRAG,
                                  lastButton, x, y };
    viewer[vrID]->input( &e );
}

//  Mouse motion callback

static void passiveMotion(int x, int y)
{
    if( viewer[vrID] == NULL ) return;

    recalc_pos( &x, &y );
    ViewerOpenGL::EventInfo e = { ViewerOpenGL::EVENT_MOUSE_MOVE,
                                  lastButton, x, y };
    viewer[vrID]->input( &e );
}

static void recalc_pos( int *x, int *y )
{
    int     height;

    height = glutGet( GLUT_WINDOW_HEIGHT );

    *y = height - *y;
}

static int load_light( arVrml97Viewer *viewer, char *buf, int n, FILE *fp)
{
    char    buf1[256], buf2[256];

    if( sscanf(buf, "%s %s", buf1, buf2) != 2 ) return -1;
    if( strcmp(buf1, "LIGHT") != 0 ) return -1;

    if( strcmp(buf2, "DIR") == 0 ) {
        float ambient;
        float intensity;
        float rgb[3];
        float dir[3];

        get_buff(buf, 256, fp);
        if( sscanf(buf, "%f", &ambient) != 1 ) return -1;
        get_buff(buf, 256, fp);
        if( sscanf(buf, "%f", &intensity) != 1 ) return -1;
        get_buff(buf, 256, fp);
        if( sscanf(buf, "%f %f %f", &rgb[0], &rgb[1], &rgb[2]) != 3 ) return -1;
        get_buff(buf, 256, fp);
        if( sscanf(buf, "%f %f %f", &dir[0], &dir[1], &dir[2]) != 3 ) return -1;

         viewer->arVrml97InsertDirLight(ambient, intensity, rgb, dir);
    }
    else if( strcmp(buf2, "POINT") == 0 ) {
        float ambient;
        float att[3];
        float rgb[3];
        float intensity;
        float loc[3];
        float radius;

        get_buff(buf, 256, fp);
        if( sscanf(buf, "%f", &ambient) != 1 ) return -1;
        get_buff(buf, 256, fp);
        if( sscanf(buf, "%f %f %f", &att[0], &att[1], &att[2]) != 3 ) return -1;
        get_buff(buf, 256, fp);
        if( sscanf(buf, "%f %f %f", &rgb[0], &rgb[1], &rgb[2]) != 3 ) return -1;
        get_buff(buf, 256, fp);
        if( sscanf(buf, "%f", &intensity) != 1 ) return -1;
        get_buff(buf, 256, fp);
        if( sscanf(buf, "%f %f %f", &loc[0], &loc[1], &loc[2]) != 3 ) return -1;
        get_buff(buf, 256, fp);
        if( sscanf(buf, "%f", &radius) != 1 ) return -1;

        viewer->arVrml97InsertPointLight(ambient, att, rgb, intensity, loc, radius);
    }
    else if( strcmp(buf2, "SPOT") == 0 ) {
        float ambient;
        float att[3];
        float beamWidth;
        float rgb[3];
        float cutOffAngle;
        float dir[3];
        float intensity;
        float loc[3];
        float radius;

        get_buff(buf, 256, fp);
        if( sscanf(buf, "%f", &ambient) != 1 ) return -1;
        get_buff(buf, 256, fp);
        if( sscanf(buf, "%f %f %f", &att[0], &att[1], &att[2]) != 3 ) return -1;
        get_buff(buf, 256, fp);
        if( sscanf(buf, "%f", &beamWidth) != 1 ) return -1;
        get_buff(buf, 256, fp);
        if( sscanf(buf, "%f %f %f", &rgb[0], &rgb[1], &rgb[2]) != 3 ) return -1;
        get_buff(buf, 256, fp);
        if( sscanf(buf, "%f", &cutOffAngle) != 1 ) return -1;
        get_buff(buf, 256, fp);
        if( sscanf(buf, "%f %f %f", &dir[0], &dir[1], &dir[2]) != 3 ) return -1;
        get_buff(buf, 256, fp);
        if( sscanf(buf, "%f", &intensity) != 1 ) return -1;
        get_buff(buf, 256, fp);
        if( sscanf(buf, "%f %f %f", &loc[0], &loc[1], &loc[2]) != 3 ) return -1;
        get_buff(buf, 256, fp);
        if( sscanf(buf, "%f", &radius) != 1 ) return -1;

        viewer->arVrml97InsertSpotLight(ambient, att, beamWidth, rgb, cutOffAngle,
                                        dir, intensity, loc, radius);
    }
    else return -1;

    return 0;
}
